// Copyright (C) 2010 Andrey Gruber (aka lamer)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include <windows.h>
#include "tags.h"

LPTAG TagsAdd(LPTAG this, wchar_t * lpText, wchar_t * lpOldText){
	LPTAG	pTag = calloc(1, sizeof(PTAG));
	LPTAG	pNext;
	if(pTag){
		wcscpy(pTag->text, lpText);
		if(lpOldText)
			wcscpy(pTag->oldText, lpOldText);
		if(!this){
			this = pTag;
		}
		else{
			pNext = this;
			while(pNext->next){
				pNext = pNext->next;
			}
			pNext->next = pTag;
			pTag->prev = pNext;
		}
	}
	return this;
}

void TagsClear(LPTAG this){
	LPTAG	pNext;

	pNext = this;
	while(pNext){
		*pNext->text = '\0';
		*pNext->oldText = '\0';
		pNext = pNext->next;
	}
}

void TagsFree(LPTAG this){
	LPTAG	pNext, pTemp;

	pNext = this;
	while(pNext){
		pTemp = pNext;
		pNext = pNext->next;
		free(pTemp);
	}
}

LPTAG TagsCopy(LPTAG tDest, LPTAG tSrc){
	LPTAG	pNext;

	if(tDest)
		TagsFree(tDest);
	tDest = NULL;
	pNext = tSrc;
	while(pNext){
		tDest = TagsAdd(tDest, pNext->text, pNext->oldText);
		pNext = pNext->next;
	}
	return tDest;
}

int TagsCompare(LPTAG this, LPTAG that){
	LPTAG	p1 = this, p2 = that;

	if((p1 && !p2) || (!p1 && p2))
		return 1;
	while(p1 && p2){
		if(_wcsicmp(p1->text, p2->text) != 0)
			return 1;
		p1 = p1->next;
		p2 = p2->next;
	}
	if((p1 && !p2) || (!p1 && p2))
		return 1;
	return 0;
}

BOOL TagsContains(LPTAG this, wchar_t * lpText){
	LPTAG	pNext = this;

	while(pNext){
		if(_wcsicmp(pNext->text, lpText) == 0)
			return TRUE;
		pNext = pNext->next;
	}
	return FALSE;
}

BOOL TagsContainsOld(LPTAG this, wchar_t * lpText){
	LPTAG	pNext = this;

	while(pNext){
		if(_wcsicmp(pNext->oldText, lpText) == 0)
			return TRUE;
		pNext = pNext->next;
	}
	return FALSE;
}

void TagsReplace(LPTAG this, wchar_t * lpNew, wchar_t * lpOld){
	LPTAG	pNext = this;

	while(pNext){
		if(_wcsicmp(pNext->text, lpOld) == 0){
			wcscpy(pNext->text, lpNew);
			return;
		}
		pNext = pNext->next;
	}
}

void TagsGetOld(LPTAG this, wchar_t * lpText, wchar_t * lpOld){
	LPTAG	pNext = this;

	while(pNext){
		if(_wcsicmp(pNext->text, lpText) == 0){
			wcscpy(lpOld, pNext->oldText);
			return;
		}
		pNext = pNext->next;
	}
}

LPTAG TagsRemove(LPTAG this, wchar_t * lpText){
	LPTAG	pNext = this, pTemp;

	while(pNext){
		if(_wcsicmp(pNext->text, lpText) == 0){
			pTemp = pNext;
			if(pNext->prev == NULL){
				//very first tag, i.e. "this"
				if(pNext->next){
					pNext->next->prev = NULL;
					this = pNext->next;
				}
				else{
					this = NULL;
				}
			}
			else if(pNext->next == NULL){
				//very last tag
				pNext->prev->next = NULL;
			}
			else{
				pNext->prev->next = pNext->next;
				pNext->next->prev = pNext->prev;
			}
			free(pTemp);
			break;
		}
		pNext = pNext->next;
	}
	return this;
}

int TagsCount(LPTAG this){
	LPTAG	pNext = this;
	int		count = 0;
	while(pNext){
		count++;
		pNext = pNext->next;
	}
	return count;
}
